﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Stimulsoft.Report;
using Stimulsoft.Report.Components;
using Stimulsoft.Report.Export;

namespace StimulsoftLicenceProblem
{
    public static class PdfExporter
    {
        /// <summary>
        /// Génère un PDF à partir des images en paramètre.
        /// </summary>
        /// <param name="streams">Images à mettre dans le PDF</param>
        /// <param name="fitToPage">Image forcée en A4</param>
        /// <returns>Le PDF généré sous forme de Stream</returns>
        public static Stream GeneratePdf(IEnumerable<Stream> streams, bool fitToPage = false, bool landscapeMode = false)
        {
            var report = ReportFactory();
            foreach (var stream in streams)
            {
                var page = new StiPage(report)
                {
                    PaperSize = PaperKind.A4,
                    Orientation = landscapeMode
                        ? StiPageOrientation.Landscape
                        : StiPageOrientation.Portrait
                };

                var stiImage = ImageFactory(stream, fitToPage);
                page.Components.Add(stiImage);
                report.Pages.Add(page);
            }

            report.RenderWithWpf(false);
            return PdfFactory(report, new MemoryStream());
        }

        /// <summary>
        /// Génère un PDF à partir des images passées en paramètre.
        /// </summary>
        /// <param name="filesName">Images à mettre dans le PDF</param>
        /// <param name="pdfFileName">Chemin complet du PDF à générer</param>
        /// <param name="fitToPage">Image forcée en A4</param>
        /// <returns>True si le traitement s'est bien passé</returns>
        public static bool GeneratePdf(IEnumerable<string> filesName, string pdfFileName, bool fitToPage = false)
        {
            var report = ReportFactory();
            foreach (var fileName in filesName)
            {
                var page = new StiPage(report)
                {
                    PaperSize = PaperKind.A4
                };

                var stiImage = ImageFactory(fileName, fitToPage);
                page.Components.Add(stiImage);
                report.Pages.Add(page);
            }

            report.RenderWithWpf(false);
            using (var pdfFile = new FileStream(pdfFileName, FileMode.Create))
            {
                PdfFactory(report, pdfFile);
                pdfFile.Close();
            }

            report.Dispose();
            return true;
        }

        private static StiReport ReportFactory()
        {
            var report = new StiReport();
            report.Pages.Clear();
            return report;
        }

        private static StiImage ImageFactory(Stream stream, bool fitToPage = false)
        {
            var stiImage = StiImageFactory(fitToPage ? StiDockStyle.Fill : StiDockStyle.Top);
            try
            {
                stiImage.Image = Image.FromStream(stream);
                return stiImage;
            }
            catch (ArgumentException e)
            {
                throw new ApplicationException("Stream format error", e);
            }
            catch (Exception defaultE)
            {
                throw new ApplicationException("Image to pdf error", defaultE);
            }
        }

        private static StiImage ImageFactory(string fileName, bool fitToPage = false)
        {
            var fs = new FileStream(fileName, FileMode.Open, FileAccess.Read);
            return ImageFactory(fs, fitToPage);
        }

        private static StiImage StiImageFactory(StiDockStyle dock = StiDockStyle.Top)
        {
            var image = new StiImage();
            image.DockStyle = dock;
            image.AspectRatio = true;
            image.CanGrow = true;
            image.CanShrink = true;
            image.Stretch = true;
            return image;
        }

        private static Stream PdfFactory(StiReport report, Stream stream)
        {
            var pdfExportService = new StiPdfExportService();
            pdfExportService.ExportPdf(report, stream);
            report.Dispose();
            return stream;
        }
    }
}
